package org.kapelle.multiblock;

import org.bukkit.Location;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockPlaceEvent;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.util.BlockVector;
import org.kapelle.multiblock.block_component.IBlockComponent;

/**
 * Listens for block placements and checks if it is part of a the multiblock structure.
 */
public abstract class MultiblockListener implements Listener {
    protected final IBlockComponent coreBlock;
    protected final BlockVector coreBlockOffset;
    protected final MultiblockStructure structure;

    public MultiblockListener(JavaPlugin plugin, MultiblockStructure structure, BlockVector coreBlockOffset){
        this.structure = structure;
        this.coreBlockOffset = coreBlockOffset;

        this.coreBlock = structure.getComponent(coreBlockOffset);

        plugin.getServer().getPluginManager().registerEvents(this, plugin);
    }

    @EventHandler
    public void onBlockPlaced(BlockPlaceEvent event){
        Location placedLocation = event.getBlockPlaced().getLocation();
        if(!event.isCancelled() && this.coreBlock.isValidBlock(placedLocation)){
            if(this.structure.isValid(placedLocation,this.coreBlockOffset)){
                callback(placedLocation);
            }
        }
    }

    /**
     * Gets called on successfully validating a multiblock
     * @param coreBlockLocation location of the core block
     */
    protected abstract void callback(Location coreBlockLocation);

}
